function SLIP_stab_bound(gamma, beta, Ehat, dim, N)

%% Copyright bel Mihly NAGY, Dra PATK, Ambrus ZELEI
%Discovery and Online Interactive Representation of the Dimensionless Parameter-Space of the Spring-Loaded Inverted Pendulum Model of Legged Locomotion Using Surface Interpolation.
%16th Internation Conference on Dynamical Systems, Theory and Aplications
%(DSTA 2021), December 6-9, 2021, On-line.
%doi.org/10.34658/9788366741201
%
%SLIP_stab_bound    plot the 3D and/or 2D boundary of the stable parameter
%region for the spring loaded inverted pendulum model
%
%   SLIP_stab_bound(gamma, beta, Ehat) plots the 2D stability boundary
%   curve at the given gamma with the custom gamma (dimensionless stiffness),
%   beta (leg angle) and Ehat (dimensionless mechanical energy) parameters
%   denoted by an +. If at the custom gamma the boundary is not calculated
%   or it does not exist, a warning will appear.
%
%   SLIP_stab_bound(gamma, beta, Ehat, dim) dim specifies the dimensions of the
%   stability diagram. If not specified, 2D diagram will be plotted.
%       0   -   2D and 3D plots
%       2   -   2D plot [x,y] axis are [beta,Ehat], black curve is the
%       stability boundary, blue is the mu=0 eigenvalue curve
%       3   -   3D plot [x,y,z] axis are [gamma,beta,Ehat], black surface is the
%       stability boundary, blue is the mu=0 eigenvalue surface
%
%   SLIP_stab_bound(gamma, beta, Ehat, dim, N) N specifies the resolution
%   of the plot. The defalt resolution, if not specified is 100.
%
%   For first trial, try this setting:
%   SLIP_stab_bound(50., 0.5, 2., 3, 100)

% Detect problem of plot dimension input
if nargin > 3
    if dim == 0 || dim == 2 || dim == 3
    else
        error('dim must be either 0, 2 or 3')
    end
end

Gamma_load=load('Stab_Bound_Data/gammas_surf_vegpont');
Gammas_sort = sort(Gamma_load.gammas);

% Detect problem of gamma input
if gamma > Gammas_sort(end)
    error(['Stability boundary is not calculated here, please choose a value ',num2str(Gammas_sort(end)),' or lower for gamma'])
elseif gamma < Gammas_sort(1)
    if nargin > 3
        if dim == 0
            warning(['There is no 2D stability region at gamma values lower than ',num2str(Gammas_sort(1)),', therefore only 3D plot will be presented'])
            dim = 3;
        elseif dim == 2
            error(['There is no 2D stability region at gamma values lower than ',num2str(Gammas_sort(1))])
        end
    else
        error(['There is no 2D stability region at gamma values lower than ',num2str(Gammas_sort(1))])
    end
end

if nargin == 5
    s = linspace(0,1,N);
else
    s = linspace(0,1);
end

if nargin > 3
    if dim == 0
        gamma_n = (gamma-Gammas_sort(1))/(Gammas_sort(end)-Gammas_sort(1));
        [~,beta_bound,ehat_bound,~,beta_0,ehat_0] = surf_stab_bound(gamma_n,s');
        
        figure()
        hold on
        grid on
        plot(beta_bound,ehat_bound,'k')
        plot(beta_0,ehat_0,'b')
        plot(beta,Ehat,'r+')
        xlabel('beta [1]')
        ylabel('Ehat [1]')
        
        [X,Y] = meshgrid(s);
        [Gamma_bound,Beta_bound,Ehat_bound,Gamma_0,Beta_0,Ehat_0] = surf_stab_bound(X,Y);
        figure()
        hold on
        surf_bound = surf(Gamma_bound,Beta_bound,Ehat_bound,'FaceAlpha',0.5);
        set(surf_bound, 'edgecolor', 'none', 'facecolor', [0 0 0], 'FaceLighting', 'gouraud')
        surf_0 = surf(Gamma_0,Beta_0,Ehat_0,'FaceAlpha',0.5);
        set(surf_0, 'edgecolor', 'none', 'facecolor', [0.3010 0.7450 0.9330], 'FaceLighting', 'gouraud')
        plot3(gamma,beta,Ehat,'r+')
        view([1,1,1])
        xlabel('gamma [1]')
        ylabel('beta [1]')
        zlabel('Ehat [1]')
    elseif dim == 2
        gamma_n = (gamma-Gammas_sort(1))/(Gammas_sort(end)-Gammas_sort(1));
        [~,beta_bound,ehat_bound,~,beta_0,ehat_0] = surf_stab_bound(gamma_n,s');
        
        figure()
        hold on
        plot(beta_bound,ehat_bound,'k')
        plot(beta_0,ehat_0,'b')
        plot(beta,Ehat,'r+')
        xlabel('beta [1]')
        ylabel('Ehat [1]')
    elseif dim == 3
        [X,Y] = meshgrid(s);
        [Gamma_bound,Beta_bound,Ehat_bound,Gamma_0,Beta_0,Ehat_0] = surf_stab_bound(X,Y);
        figure()
        hold on
        surf_bound = surf(Gamma_bound,Beta_bound,Ehat_bound,'FaceAlpha',0.5);
        set(surf_bound, 'edgecolor', 'none', 'facecolor', [0 0 0], 'FaceLighting', 'gouraud')
        surf_0 = surf(Gamma_0,Beta_0,Ehat_0,'FaceAlpha',0.5);
        set(surf_0, 'edgecolor', 'none', 'facecolor', [0.3010 0.7450 0.9330], 'FaceLighting', 'gouraud')
        plot3(gamma,beta,Ehat,'r+')
        view([1,1,1])
        xlabel('gamma [1]')
        ylabel('beta [1]')
        zlabel('Ehat [1]')
    end
else
    gamma_n = (gamma-Gammas_sort(1))/(Gammas_sort(end)-Gammas_sort(1));
    [~,beta_bound,ehat_bound,~,beta_0,ehat_0] = surf_stab_bound(gamma_n,s');
    
    figure()
    hold on
    grid on
    plot(beta_bound,ehat_bound,'k')
    plot(beta_0,ehat_0,'b')
    plot(beta,Ehat,'r+')
    xlabel('beta [1]')
    ylabel('Ehat [1]')
end

%%

function [Gamma,Beta,Ehat,Gamma_0,Beta_0,Ehat_0] = surf_stab_bound(X,Y)
Beta = zeros(size(Y));
Ehat = zeros(size(Y));

if length(X)== 1
    X = X*ones(size(Y,1),1);
end

Gamma_load=load('Stab_Bound_Data/gammas_surf_vegpont');

Gammas_sort = sort(Gamma_load.gammas);

Gammas_norm = (Gammas_sort-Gammas_sort(1))/(Gammas_sort(end)-Gammas_sort(1));

Gamma = Gammas_sort(1)+X*(Gammas_sort(end)-Gammas_sort(1));
for i = 1:size(X,2)
    flag_veg = 0;
    k = 2;
    while X(1,i) > Gammas_norm(k)
        k = k+1;
    end
    
    if k == 2
        A_m=load('Stab_Bound_Data/spline_veg');
        flag_veg = 1;
    else
        A_m=load(['Stab_Bound_Data/spline_gamma_',num2str(floor(Gammas_sort(k-1))),'_',num2str(100*(Gammas_sort(k-1)-floor(Gammas_sort(k-1))))]);
        x1 = Gammas_norm(k-1);
    end
    A_p=load(['Stab_Bound_Data/spline_gamma_',num2str(floor(Gammas_sort(k))),'_',num2str(100*(Gammas_sort(k)-floor(Gammas_sort(k))))]);
    x2 = Gammas_norm(k);
    
    %     [x1,x2]
    for j = 1:size(Y,1)
        k = 1;
        while Y(j,i) > A_m.sp_betas(k,4)
            k = k+1;
        end
        fb1 = polyval(A_m.sp_betas(k,1:3),Y(j,i));
        fE1 = polyval(A_m.sp_Ebars(k,1:3),Y(j,i));
        if flag_veg == 1
            fg1 = polyval(A_m.sp_gammas(k,1:3),Y(j,i));
            x1 = (fg1-Gammas_sort(1))/(Gammas_sort(end)-Gammas_sort(1));
        end
        
        if x1>X(1,i)
            Gamma(j,i)=nan;
            Beta(j,i)=nan;
            Ehat(j,i)=nan;
        else
            k = 1;
            while Y(j,i) > A_p.sp_betas(k,4)
                k = k+1;
            end
            fb2 = polyval(A_p.sp_betas(k,1:3),Y(j,i));
            fE2 = polyval(A_p.sp_Ebars(k,1:3),Y(j,i));
            
            
            df1 = (fb2-fb1)/(x2-x1);
            Beta_eh = [(-fb1 + fb2 + df1.*(x1 - x2))./(x1 - x2).^2;
                (2*fb1.*x1 - 2*fb2.*x1 + df1.*(-x1.^2 + x2.^2))./(x1 - x2).^2;
                (fb2.*x1.^2 + df1.*x1.*(x1 - x2).*x2 + fb1.*x2.*(-2*x1 + x2))./(x1 - x2).^2]';
            
            df1 = (fE2-fE1)/(x2-x1);
            E_eh = [(-fE1 + fE2 + df1.*(x1 - x2))./(x1 - x2).^2;
                (2*fE1.*x1 - 2*fE2.*x1 + df1.*(-x1.^2 + x2.^2))./(x1 - x2).^2;
                (fE2.*x1.^2 + df1.*x1.*(x1 - x2).*x2 + fE1.*x2.*(-2*x1 + x2))./(x1 - x2).^2]';
            
            Beta(j,i) = polyval(Beta_eh,X(1,i));
            Ehat(j,i) = polyval(E_eh,X(1,i));
        end
    end
end

[Gamma_0,Beta_0,Ehat_0] = surf_0(X,Y);

function [Gamma,Beta,Ehat] = surf_0(X,Y)
Beta = zeros(size(Y));
Ehat = zeros(size(Y));

Gamma_load=load('Stab_Bound_Data/gammas_surf_vegpont');

Gammas_sort = sort(Gamma_load.gammas);

Gammas_norm = (Gammas_sort-Gammas_sort(1))/(Gammas_sort(end)-Gammas_sort(1));

Gamma = Gammas_sort(1)+X*(Gammas_sort(end)-Gammas_sort(1));
bad_index = 0;
for i = 1:size(X,2)
    k = 2;
    while X(1,i) > Gammas_norm(k)
        k = k+1;
    end
    
    if floor(Gammas_sort(k-1)) > 9
        
        A_m=load(['Stab_Bound_Data/spline_gamma_',num2str(floor(Gammas_sort(k-1))),'_',num2str(100*(Gammas_sort(k-1)-floor(Gammas_sort(k-1)))),'_0']);
        x1 = Gammas_norm(k-1);
        A_p=load(['Stab_Bound_Data/spline_gamma_',num2str(floor(Gammas_sort(k))),'_',num2str(100*(Gammas_sort(k)-floor(Gammas_sort(k)))),'_0']);
        x2 = Gammas_norm(k);
        
        %     [x1,x2]
        for j = 1:size(Y,1)
            k = 1;
            while Y(j,i) > A_m.sp_betas(k,4)
                k = k+1;
            end
            fb1 = polyval(A_m.sp_betas(k,1:3),Y(j,i));
            fE1 = polyval(A_m.sp_Ebars(k,1:3),Y(j,i));
            
            k = 1;
            while Y(j,i) > A_p.sp_betas(k,4)
                k = k+1;
            end
            fb2 = polyval(A_p.sp_betas(k,1:3),Y(j,i));
            fE2 = polyval(A_p.sp_Ebars(k,1:3),Y(j,i));
            
            
            df1 = (fb2-fb1)/(x2-x1);
            Beta_eh = [(-fb1 + fb2 + df1.*(x1 - x2))./(x1 - x2).^2;
                (2*fb1.*x1 - 2*fb2.*x1 + df1.*(-x1.^2 + x2.^2))./(x1 - x2).^2;
                (fb2.*x1.^2 + df1.*x1.*(x1 - x2).*x2 + fb1.*x2.*(-2*x1 + x2))./(x1 - x2).^2]';
            
            df1 = (fE2-fE1)/(x2-x1);
            E_eh = [(-fE1 + fE2 + df1.*(x1 - x2))./(x1 - x2).^2;
                (2*fE1.*x1 - 2*fE2.*x1 + df1.*(-x1.^2 + x2.^2))./(x1 - x2).^2;
                (fE2.*x1.^2 + df1.*x1.*(x1 - x2).*x2 + fE1.*x2.*(-2*x1 + x2))./(x1 - x2).^2]';
            
            Beta(j,i) = polyval(Beta_eh,X(1,i));
            Ehat(j,i) = polyval(E_eh,X(1,i));
        end
    else
        bad_index = i;
    end
end

Gamma = Gamma(:,bad_index+1:end);
Beta = Beta(:,bad_index+1:end);
Ehat = Ehat(:,bad_index+1:end);

